/**
 * A BLE client example that is rich in capabilities.
 * There is a lot new capabilities implemented.
 * author unknown
 * updated by chegewara
 */

#include "BLEDevice.h"
//#include "BLEScan.h"
#include "DFRobot_GDL.h"
#include "Icon.h"

#define TFT_DC D2
#define TFT_CS D6
#define TFT_RST D3
#define TFT_BL D13
#define TFT_SD D7

DFRobot_ST7789_240x320_HW_SPI screen(/*dc=*/TFT_DC, /*cd=*/TFT_CS, /*rst=*/TFT_RST);

#define RXD2 17 //communication using Serial1
#define TXD2 16 //communication using Serial1

//connection to local wifi to send data to Node-Red:
const char* ssid = "Edimax";        //WiFi SSID to connect to send data to Node-Red
const char* pass = "Antonia2011!";  //WiFi pass to connect to send data to Node-Red
const char* mqtt_client_id = "test-client";                //MQTT client ID used by Node-red
const char* mqtt_user = "mihai";                     //MQTT user for Node-Red; see Node-Red documentation
const char* mqtt_pass = "mihai";                     //MQTT pass for Node-Red; see Node-Red documentation
const char* mqtt_host = "192.168.0.35";                     //MQTT host for Node-Red; see Node-Red documentation
char finalTopic_wifi[200];              //used to send the complete string to Node-Red 

// The remote service we wish to connect to.
static BLEUUID serviceUUID("4fafc201-1fb5-459e-8fcc-c5c9c331914b");

// The characteristic of the remote service we are interested in.
static BLEUUID charUUID_bme("beb5483e-36e1-4688-b7f5-ea07361b26a8");
static BLEUUID charUUID_light("f944b8d6-8558-44ca-8efa-b6d1a0e47480");

static boolean doConnect = false;
static boolean connected = false;
static boolean doScan = false;
String Temp = "";
String Humi = "";
String Pres = "";
String Light = "";

int w = screen.width();
int h = screen.height();
int16_t color_ble = 0xffffff; //0xffffff = white colour
int pressureAtm_mmHg;
int temperatureCTempIntiger;
int humidityIntiger;
int itu;
int lightIntiger;
int uvIntiger;

int temp_current   = 0;
int temp_previous  = -1;
int humi_current   = 0;
int humi_previous  = -1;
int pres_current   = 0;
int pres_prevoius  = -1;
int itu_current    = 0;
int itu_previous   = -1;
int light_current  = 0;
int light_previous = -1;
int uv_current     = 0;
int uv_previous    = -1;

BLERemoteCharacteristic* pRemoteChar_bme;
BLERemoteCharacteristic* pRemoteChar_light;

static BLEAdvertisedDevice *myDevice;

//used in case of server characteristscs are set to Notify:
static void notifyCallback(BLERemoteCharacteristic *pBLERemoteCharacteristic, 
                            uint8_t *pData, 
                            size_t length, 
                            bool isNotify) {
  if(pBLERemoteCharacteristic->getUUID().toString() == charUUID_bme.toString()) {      
    Serial.print("Temp is: ");  
    for(int i=0; i<length; i++)
    {
      Temp +=(char)pData[i];     
    }

    Serial.print(Temp);
    Serial.println(" C"); 
    Temp = ""; //reset the display string           
  }

  else if(pBLERemoteCharacteristic->getUUID().toString() == charUUID_light.toString()) {
    Serial.print("Light is: ");
    
    for(int i=0; i<length; i++)
    {
      Light +=(char)pData[i];     
    }

//      Serial.print((char)pData[i]);
    Serial.print(Light);
    Serial.println(" %Rh"); 
    Light = ""; //reset the display string  
  }
}

class MyClientCallback : public BLEClientCallbacks {
  void onConnect(BLEClient *pclient) {}

  void onDisconnect(BLEClient *pclient) {
    connected = false;
    Serial.println("onDisconnect");
  }
};

bool connectToServer() {
  Serial.print("Forming a connection to ");
  Serial.println(myDevice->getAddress().toString().c_str());

  BLEClient *pClient = BLEDevice::createClient();
  Serial.println(" - Created client");

  pClient->setClientCallbacks(new MyClientCallback());

  // Connect to the remove BLE Server.
  pClient->connect(myDevice);  // if you pass BLEAdvertisedDevice instead of address, it will be recognized type of peer device address (public or private)
  Serial.println(" - Connected to server");
  pClient->setMTU(517);  //set client to request maximum MTU from server (default is 23 otherwise)

  // Obtain a reference to the service we are after in the remote BLE server.
  BLERemoteService *pRemoteService = pClient->getService(serviceUUID);
  if (pRemoteService == nullptr) {
    Serial.print("Failed to find our service UUID: ");
    Serial.println(serviceUUID.toString().c_str());
    pClient->disconnect();
    return false;
  }
  Serial.println(" - Found our service");

  connected = true;

  if(connectCharacteristic(pRemoteService, charUUID_bme) == false)
    connected = false;
  else if (connectCharacteristic(pRemoteService, charUUID_light) == false)
    connected = false;
    
  if(connected == false) 
  {
    pClient->disconnect();
    Serial.print("At least one charcacteristic UUID not found");
    return false;
  }

  pRemoteChar_bme = pRemoteService->getCharacteristic(charUUID_bme);
  pRemoteChar_light = pRemoteService->getCharacteristic(charUUID_light);
    
  return true;
}

bool connectCharacteristic(BLERemoteService* pRemoteService, BLEUUID l_charUUID)
{  
  // Obtain a reference to the characteristic in the service of the remote BLE server.
  BLERemoteCharacteristic *pRemoteCharacteristic;
  pRemoteCharacteristic = pRemoteService->getCharacteristic(l_charUUID);
  if (pRemoteCharacteristic == nullptr) {
    Serial.print("Failed to find our characteristic UUID: ");
    Serial.println(l_charUUID.toString().c_str());
    return false;
  }
  Serial.println(" - Found our characteristic");

  // Read the value of the characteristic.
  if (pRemoteCharacteristic->canRead()) {
    String value = pRemoteCharacteristic->readValue();
//    std::string value_str = pRemoteChar_light->readValue(); // Citim valoarea în std::string
//    String value = String(value_str.c_str()); // Convertim std::string în String
    Serial.print("The characteristic value was: ");
    Serial.println(value.c_str());
  }

  if (pRemoteCharacteristic->canNotify()) {
    pRemoteCharacteristic->registerForNotify(notifyCallback);
  }

  return true;
  
}

/**
 * Scan for BLE servers and find the first one that advertises the service we are looking for.
 */
class MyAdvertisedDeviceCallbacks : public BLEAdvertisedDeviceCallbacks {
  /**
   * Called for each advertising BLE server.
   */
  void onResult(BLEAdvertisedDevice advertisedDevice) {
    Serial.print("BLE Advertised Device found: ");
    Serial.println(advertisedDevice.toString().c_str());

    // We have found a device, let us now see if it contains the service we are looking for.
    if (advertisedDevice.haveServiceUUID() && advertisedDevice.isAdvertisingService(serviceUUID)) {

      BLEDevice::getScan()->stop();
      myDevice = new BLEAdvertisedDevice(advertisedDevice);
      doConnect = true;
      doScan = true;

    }  // Found our server
  }    // onResult
};     // MyAdvertisedDeviceCallbacks

void display_initial_data_on_TFT()
{
  screen.setTextWrap(false);
  screen.setRotation(0);
  screen.fillScreen(COLOR_RGB565_BLACK);  //fill the entire THT fith "black" colour
  
  screen.setTextColor(COLOR_RGB565_WHITE); //set the text colour
  
  screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
  screen.setTextSize(2);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
  screen.setCursor(64, 28);                //set cursor
  screen.print("__,_");                    //default value for temperature
  screen.drawCircle(190, 10, 4, COLOR_RGB565_WHITE); 
  screen.drawCircle(190, 10, 5, COLOR_RGB565_WHITE); 
  screen.drawCircle(190, 10, 6, COLOR_RGB565_WHITE); 
  screen.setCursor(205,28);
  screen.print("C");                       //sign for temperature

  screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
  screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
  screen.setCursor(0, 80);                 //set cursor
  screen.print("__,_");                    //default value for humidity
  screen.setCursor(60, 80);                //set cursor
  screen.print("%Rh");                     //sign for humidity

  screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
  screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
  screen.setCursor(120, 80);               //set cursor
  screen.print("ITU");                     //sign for humidity
  screen.setCursor(170, 80);               //set cursor
  screen.print("__");                      //default value for ITU  

  screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
  screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
  screen.setCursor(45, 128);               //set cursor
  screen.print("___");                     //default value for ATMpressure
  screen.setCursor(90, 128);               //set cursor
  screen.print("mmHg");                    //sign for ATMpressure

  screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
  screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
  screen.setCursor(165, 128);              //set cursor
  screen.print("UV");                      //sign/name for UV
  screen.setCursor(200, 128);              //set cursor
  screen.print("_");                       //default value for UV 
}

void read_data_from_serial1()
{
    while (true) 
    {   
      if (Serial1.available() > 0)
       {     
        String citire = Serial1.readString();
        Serial.print("Response: ");
        Serial.println(citire);
        break; //out from loop after message is received from serial
      }
    }
}

//connect ESP32 used fot AT commands to local WiFi network:
void connect_to_local_wifi()
{
  //Verify disponibility of ESP8266-01:
    Serial1.println("AT");
    Serial.println("AT");
    read_data_from_serial1();
    delay(1000); 
  //Reset ESP8266-01:
    Serial1.println("AT+RST");
    Serial.println("AT+RST");    
    read_data_from_serial1();
    delay(2000); 
  //Set ESP8266 mode 3 = access point and station mode   
    Serial1.println("AT+CWMODE=3");
    Serial.println("AT+CWMODE=3");    
    read_data_from_serial1();
    delay(5000);
    String command = "AT+CWJAP=\"";
    command += ssid;
    command += "\",\"";
    command += pass;
    command += "\"";
    Serial1.println(command);
    Serial.println(command);    
    read_data_from_serial1();
    delay(5000);
    Serial1.println("AT+CIFSR");           //connect to AP
    Serial.println("AT+CIFSR");           //connect to AP    
    read_data_from_serial1();
    delay(100); 
}

void connect_to_mqtt_broker()
{
    sprintf(finalTopic_wifi, "AT+MQTTUSERCFG=0,1,\"%s\",\"%s\",\"%s\",0,0,\"\"", mqtt_client_id, mqtt_user, mqtt_pass);
    Serial1.println(finalTopic_wifi);  
    Serial.println(finalTopic_wifi); 
    read_data_from_serial1(); 
    delay(500);

    memset(finalTopic_wifi, '\0', sizeof(finalTopic_wifi));       //reset the string   
     
    sprintf(finalTopic_wifi, "AT+MQTTCONN=0,\"%s\",1883,1", mqtt_host);
    Serial1.println(finalTopic_wifi);
    Serial.println(finalTopic_wifi);
    read_data_from_serial1();
    delay(500);    
}

void send_mqtt_data_to_node_red()
{
    sprintf(finalTopic_wifi, "AT+MQTTUSERCFG=0,1,\"%s\",\"%s\",\"%s\",0,0,\"\"", mqtt_client_id, mqtt_user, mqtt_pass);
    Serial1.println(finalTopic_wifi);  
    Serial.println(finalTopic_wifi); 
    read_data_from_serial1(); 
    delay(500);

    memset(finalTopic_wifi, '\0', sizeof(finalTopic_wifi));       //reset the string   
     
    sprintf(finalTopic_wifi, "AT+MQTTCONN=0,\"%s\",1883,1", mqtt_host);
    Serial1.println(finalTopic_wifi);
    Serial.println(finalTopic_wifi);
    read_data_from_serial1();
    delay(500);   

    Serial.println("Send data from analog sensors: ");

    //send Mmqtt messages with data from temperature sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/temperature\",\"" + String(temperatureCTempIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/temperature\",\"" + String(temperatureCTempIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);

    //send Mmqtt messages with data from humidity sensor:    
    Serial1.println("AT+MQTTPUB=0,\"weather/humidity\",\"" + String(humidityIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/humidity\",\"" + String(humidityIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);

    //send Mmqtt messages with data from pressureAtm_mmHg sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/pressureAtm\",\"" + String(pressureAtm_mmHg) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/pressureAtm\",\"" + String(pressureAtm_mmHg) + "\",1,0");
    read_data_from_serial1();
    delay(500);

    //send Mmqtt messages with data from light sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/light\",\"" + String(lightIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/light\",\"" + String(lightIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);

    //send Mmqtt messages with data from UV sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/UVsensor\",\"" + String(uvIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/UVsensor\",\"" + String(uvIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);    
}

//**********************************************************************************
//*********************** This is the Arduino setup function************************
//**********************************************************************************
void setup() {
  Serial.begin(115200);

  Serial1.begin(9600, SERIAL_8N1, RXD2, TXD2); //comunication via Serial1
  Serial.println("Comunication with ESP8266-01 is open...");  

  connect_to_local_wifi(); //connect to local WiFi
  connect_to_mqtt_broker();//connect to MQTT broker
  
  screen.begin();

  display_initial_data_on_TFT(); //display the initial data on TFT:
  
  Serial.println("Starting Arduino BLE Client application...");
  BLEDevice::init("");
 
  // Retrieve a Scanner and set the callback we want to use to be informed when we
  // have detected a new device.  Specify that we want active scanning and start the
  // scan to run for 5 seconds.
  BLEScan *pBLEScan = BLEDevice::getScan();
  pBLEScan->setAdvertisedDeviceCallbacks(new MyAdvertisedDeviceCallbacks());
  pBLEScan->setInterval(1349);
  pBLEScan->setWindow(449);
  pBLEScan->setActiveScan(true);
  pBLEScan->start(5, false);
}  // End of setup.

//**********************************************************************************
//*********************** This is the Arduino main loop function********************
//**********************************************************************************
void loop() {

  // If the flag "doConnect" is true then we have scanned for and found the desired
  // BLE Server with which we wish to connect.  Now we connect to it.  Once we are
  // connected we set the connected flag to be true.
  if (doConnect == true) {
    if (connectToServer()) {
      Serial.println("We are now connected to the BLE Server.");
      //display BLE sign:
      screen.drawXBitmap(/*x=*/0,/*y=*/0,/*bitmap gImage_Bitmap=*/gImage[0],/*w=*/32,/*h=*/32,color_ble);
    } else {
      Serial.println("We have failed to connect to the server; there is nothing more we will do.");    
    }
    doConnect = false;
  }

  // If we are connected to a peer BLE Server, update the characteristic each time we are reached
  // with the current time since boot.
  if (connected) 
  {
    String bmeValue = pRemoteChar_bme->readValue();
//    std::string bmeValue_str = pRemoteChar_light->readValue(); // Citim valoarea în std::string
//    String bmeValue = String(bmeValue_str.c_str()); // Convertim std::string în String
    // Convert String "bmeValue" into an characters array (char[]):
    char bmeReceivedString[bmeValue.length() + 1];
    bmeValue.toCharArray(bmeReceivedString, bmeValue.length() + 1);

    String ltrValue = pRemoteChar_light->readValue();
//    std::string ltrValue_str = pRemoteChar_light->readValue(); // Citim valoarea în std::string
//    String ltrValue = String(ltrValue_str.c_str()); // Convertim std::string în String
    // Convert String "ltrValue" into an characters array (char[]): 
    char ltrReceivedString[ltrValue.length() + 1];
    ltrValue.toCharArray(ltrReceivedString, ltrValue.length() + 1);       

    char temperatureCTemp[10];
    char humidityTemp[10];
    char pressureAtmTemp[10];
    char lightTemp[10];
    char uvTemp[5];
    char* token;
    char temperatureExtracted[5];
    char humidityExtracted[5];
    char pressureAtmExtracted[4];
    char pressureAtm_mmHgString[4];
    char uvIntigerString[1];
    char lightExtracted[10];
    char uvExtracted[3];

//extract data from received strings:
    //extract temperature from received string:
    token = strtok(bmeReceivedString, ",");
    if (token != NULL) {
      strcpy(temperatureCTemp, token);
      String temperatureCTempStr = String(temperatureCTemp);                                //convert chat array in String
      int decimalIndexT = temperatureCTempStr.indexOf('.');                                 //fing the decimal point in received string
      String temperatureCTemp_integerPart = temperatureCTempStr.substring(0, decimalIndexT);//extract the integer part
      temperatureCTempIntiger = temperatureCTemp_integerPart.toInt();                      //convert the integer part in number
     
      strncpy(temperatureExtracted, &temperatureCTemp[1], 4);
      temperatureExtracted[4] = '\0';
    }

    //extract himidity from received string:
    token = strtok(NULL, ",");
    if (token != NULL) {
      strcpy(humidityTemp, token);
      String humidityStr = String(humidityTemp); //convert chat array in String
      int decimalIndexH = humidityStr.indexOf("."); //fing the decimal point in received string
      String humidity_integerPart = humidityStr.substring(0, decimalIndexH);//extract the integer part
      humidityIntiger = humidity_integerPart.toInt();
      
      strncpy(humidityExtracted, &humidityTemp[1], 4);
      humidityExtracted[4] = '\0';
    }

    //extract atmospheric pressure from received string:
    token = strtok(NULL, ",");
    if (token != NULL) {
      strcpy(pressureAtmTemp, token);
      String pressureAtmStr = String(pressureAtmTemp);                             //convert char array in String
      int decimalIndexP = pressureAtmStr.indexOf('.');                             //fing the decimal point in received string
      String pressureAtm_interegPart = pressureAtmStr.substring(0, decimalIndexP); //extract the integer part
      int pressureAtmInteger = pressureAtm_interegPart.toInt();                    //convert the integer part in number
      pressureAtm_mmHg = (int)(pressureAtmInteger * 0.75);                         //comvert pressute to mmHg
      sprintf(pressureAtm_mmHgString, "%d", pressureAtm_mmHg);                     //convert value to string for TFT
    }

    //extract light from received string:
    token = strtok(ltrReceivedString, ",");
    if (token != NULL) {
      strcpy(lightTemp, token);
      String lightTempStr = String(lightTemp);                                //convert chat array in String
      int decimalIndexT = lightTempStr.indexOf('.');                                 //fing the decimal point in received string
      String lightTemp_integerPart = lightTempStr.substring(0, decimalIndexT);//extract the integer part
      lightIntiger = lightTemp_integerPart.toInt();                      //convert the integer part in number
     
      strncpy(lightExtracted, &lightTemp[1], 4);
      lightExtracted[4] = '\0';
    }

    //extract UV from received string:
    token = strtok(NULL, ",");
    if (token != NULL) {
      strcpy(uvTemp, token);
      String uvStr = String(uvTemp); //convert chat array in String
      int decimalIndexH = uvStr.indexOf("."); //fing the decimal point in received string
      String uv_integerPart = uvStr.substring(0, decimalIndexH);//extract the integer part
      uvIntiger = uv_integerPart.toInt();
      
      strncpy(uvExtracted, &uvTemp[1], 4);
      uvExtracted[4] = '\0';
    }

//calcul ITU (temperature-humidity index):
  itu = (temperatureCTempIntiger*1.8 + 32) - (0.55 - 0.0055*humidityIntiger) * ((temperatureCTempIntiger*1.8 + 32)-58);

//***
#if 0 //used to "ignore" the code until "hashtag end":
//*****************************************************************************************************
//******** Send data to serial, display and MQTT at value change **************************************
//*****************************************************************************************************
//check and update values on display, display and MQTT only if differrence from last received value:
//check data for temperature:
  temp_current = temperatureCTempIntiger;    //"read" integer value of temp
  if (temp_current !=temp_previous)
  {
    //send temperature data to serial: 
    Serial.print("Temperature: ");
    Serial.print(temperatureCTemp);
    Serial.println(" ºC"); 

    //display temperature data to TFT:
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(2);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(64, 0, 120, 40, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(64, 28);                //set cursor  
    screen.print(temperatureExtracted);      //default value/sign for temperature

    //send Mmqtt messages with data from temperature sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/temperature\",\"" + String(temperatureCTempIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/temperature\",\"" + String(temperatureCTempIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);   

    temp_previous = temp_current;
  }

//check data for humidity:
    humi_current = humidityIntiger;          //"read" integer value of humidity
    if (humi_current !=humi_previous)
    {
    //send humidity data to serial: 
    Serial.print("Humidity: ");
    Serial.print(humidityTemp);
    Serial.println(" %");    

    //display humidity data to TFT:
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(0, 66, 56, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(0, 80);                 //set cursor
    screen.print(humidityExtracted);         //default value/sign for humidity

    //send Mmqtt messages with data from humidity sensor:    
    Serial1.println("AT+MQTTPUB=0,\"weather/humidity\",\"" + String(humidityIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/humidity\",\"" + String(humidityIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);
    
    humi_previous = humi_current;         
    }

//check data for atmospheric pressure:
    pres_current = pressureAtm_mmHg;         //"read" integer value of atm pressure   
    if (pres_current != pres_prevoius)
    {
    //send atmospheric pressure data to serial: 
    Serial.print("Pressure: ");
    Serial.print(pressureAtmTemp);
    Serial.println(" hPa"); 

    Serial.print("Pressure: ");
    Serial.print(pressureAtm_mmHg);
    Serial.println(" mmHg");

    //display atmospheric pressure data to TFT:    
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(45, 114, 42, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(45, 128);               //set cursor
    sprintf(pressureAtm_mmHgString, "%d", pressureAtm_mmHg);   //convert value to string for TFT
    screen.print(pressureAtm_mmHgString);    //display value

    //calcul and display barometer pictograms:
    if (pressureAtm_mmHg < 750) //rain icon
      {
//        screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[4],/*w=*/32,/*h=*/32,color_ble);    
      }
    else if ((pressureAtm_mmHg >=750) && (pressureAtm_mmHg<765)) //cloud icon
      {
//        screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[2],/*w=*/32,/*h=*/32,color_ble);     
      }
    else if (pressureAtm_mmHg >=765) //sun icon
      {
 //       screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[1],/*w=*/32,/*h=*/32,color_ble);    
      }

    //send Mmqtt messages with data from pressureAtm_mmHg sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/pressureAtm\",\"" + String(pressureAtm_mmHg) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/pressureAtm\",\"" + String(pressureAtm_mmHg) + "\",1,0");
    read_data_from_serial1();
    delay(500);

    pres_prevoius = pres_current;
    }

//check data for ITU:
    itu_current = itu;                       //"read" integer value of itu 
    if (itu_current != itu_previous)
    {
    //send ITU data to serial: 
    Serial.print("ITU value: ");
    Serial.print(itu);
    Serial.println(" %");

    //display Temperature-Humidity Index (ITU) data to TFT:
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(170, 66, 42, 20, COLOR_RGB565_BLACK);     //"delete" the old value
    screen.setCursor(170, 80);               //set cursor
    screen.print(itu);                       //default value/sign for ATMPressure

    itu_previous = itu_current;     
    }

//check data for light:
    light_current = lightIntiger;            //"read" integer value of light
    if (light_current != light_previous)
    {
    //send light data to serial:
    Serial.print("Light read is: ");
    Serial.print(lightTemp);
    Serial.println(" lux");

    //send Mmqtt messages with data from light sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/light\",\"" + String(lightIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/light\",\"" + String(lightIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500);
    
    light_previous = light_current;        
    }

//check data for UV:
    uv_current = uvIntiger;                  //"read" integer value of light
    if (uv_current != uv_previous)
    {
    //send UV data to serial:
    Serial.print("UV index is: ");
    Serial.print(uvTemp);
    Serial.println(" index");  

    //display UV data to TFT:
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(200, 114, 14, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(200, 128);              //set cursor
    sprintf(uvIntigerString, "%d", uvIntiger);                 //convert value to string for TFT
    screen.print(uvIntigerString);           //display value        

    //send Mmqtt messages with data from UV sensor:
    Serial1.println("AT+MQTTPUB=0,\"weather/UVsensor\",\"" + String(uvIntiger) + "\",1,0");
    Serial.println("AT+MQTTPUB=0,\"weather/UVsensor\",\"" + String(uvIntiger) + "\",1,0");
    read_data_from_serial1();
    delay(500); 
    
    uv_previous = uv_current;  
    }
//*****************************************************************************************************
#endif
//***

//*****************************************************************************************************
//******** Send data to serial, display and MQTT continuously, at 10 seconds **************************
//*****************************************************************************************************
//send data to serial: 
    Serial.print("Temperature: ");
    Serial.print(temperatureCTemp);
    Serial.println(" ºC");
          
    Serial.print("Humidity: ");
    Serial.print(humidityTemp);
    Serial.println(" %");
       
    Serial.print("Pressure: ");
    Serial.print(pressureAtmTemp);
    Serial.println(" hPa"); 

    Serial.print("Pressure: ");
    Serial.print(pressureAtm_mmHg);
    Serial.println(" mmHg"); 
   
    Serial.print("ITU value: ");
    Serial.print(itu);
    Serial.println(" %");

//    String txLightValue = pRemoteChar_light->readValue();
    Serial.print("Light read is: ");
    Serial.print(lightTemp);
    Serial.println(" lux");

    Serial.print("UV index is: ");
    Serial.print(uvTemp);
    Serial.println(" index");
    
    Serial.println("");

    //display data to TFT:
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(2);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(64, 0, 120, 40, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(64, 28);                //set cursor  
    screen.print(temperatureExtracted);      //default value/sign for temperature

    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(0, 66, 56, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(0, 80);                 //set cursor
    screen.print(humidityExtracted);         //default value/sign for humidity

    //display Temperature-Humidity Index (ITU):
    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(170, 66, 42, 20, COLOR_RGB565_BLACK);     //"delete" the old value
    screen.setCursor(170, 80);               //set cursor
    screen.print(itu);                       //default value/sign for ATMPressure

    //calcul and display barometer pictograms:
    if (pressureAtm_mmHg < 750) //rain icon
      {
        screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[4],/*w=*/32,/*h=*/32,color_ble);    
      }
    else if ((pressureAtm_mmHg >=750) && (pressureAtm_mmHg<765)) //cloud icon
     {
        screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[2],/*w=*/32,/*h=*/32,color_ble);     
      }
    else if (pressureAtm_mmHg >=765) //sun icon
      {
        screen.drawXBitmap(/*x=*/0,/*y=*/100,/*bitmap gImage_Bitmap=*/gImage[1],/*w=*/32,/*h=*/32,color_ble);    
      }

    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(45, 114, 42, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(45, 128);               //set cursor
    sprintf(pressureAtm_mmHgString, "%d", pressureAtm_mmHg);   //convert value to string for TFT
    screen.print(pressureAtm_mmHgString);    //display value

    screen.setFont(&FreeMonoBold12pt7b);     //set the font for text
    screen.setTextSize(1);                   //set text size: 1 = 14x14 pixels; 2 = 28x28 pixels; available for font "FreeMonoBold12pt7b"     
    screen.fillRect(200, 114, 14, 20, COLOR_RGB565_BLACK); //"delete" the old value
    screen.setCursor(200, 128);              //set cursor
    sprintf(uvIntigerString, "%d", uvIntiger);                 //convert value to string for TFT
    screen.print(uvIntigerString);           //display value

    send_mqtt_data_to_node_red();

    delay(10000); //delay 10 seconds befor data transmissions
//*****************************************************************************************************
    
    
    // Set the characteristic's value to be the array of bytes that is actually a string.
//    pRemoteCharacteristic->writeValue(newValue.c_str(), newValue.length());
  } else if (doScan) {
    display_initial_data_on_TFT();
    screen.fillRect(0, 0, 32, 32, COLOR_RGB565_BLACK); //"delete" the old value  
    BLEDevice::getScan()->start(0);  // this is just example to start scan after disconnect, most likely there is better way to do it in arduino     
  }

  delay(1000);  // Delay a second between loops.
}  // End of loop
