/*
  Based on Neil Kolban example for IDF: https://github.com/nkolban/esp32-snippets/blob/master/cpp_utils/tests/BLE%20Tests/SampleNotify.cpp
  Ported to Arduino ESP32 by Evandro Copercini
  updated by chegewara and MoThunderz
*/
#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>

#include <Wire.h>

#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>

//#include <BH1750.h>

#include <LTR390.h>


//********************used for BLE communication:
BLEServer* pServer = NULL;
BLECharacteristic* pCharacteristic_bme = NULL;
BLECharacteristic* pCharacteristic_light = NULL;

BLEDescriptor *pDescr_bme;
BLEDescriptor *pDescr_light;

BLE2902 *pBLE2902_bme;
BLE2902 *pBLE2902_light;

// See the following for generating UUIDs: https://www.uuidgenerator.net/

//BLE server name
#define bleServerName "Environmental station"

#define SERVICE_UUID               "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID_bme    "beb5483e-36e1-4688-b7f5-ea07361b26a8"
#define CHARACTERISTIC_UUID_light  "f944b8d6-8558-44ca-8efa-b6d1a0e47480"

bool deviceConnected    = false;
bool oldDeviceConnected = false;
uint32_t value = 0;

//******************** used for read BME280:
#define temperatureCelsius //Default Temperature is in Celsius

Adafruit_BME280 bme;       // I2C
float temp;
float humi;
float pres;

//******************** used for read BH1750:
//BH1750 lightMeter(0x23); 
//float lux;

//******************** used for read LTR390:
#define I2C_ADDRESS 0x53
LTR390 ltr390(I2C_ADDRESS);
float lux;
float uv;

//******************** used to initialize the onboard led:
int led = 15;


class MyServerCallbacks: public BLEServerCallbacks {
    void onConnect(BLEServer* pServer) {
      deviceConnected = true;
    };

    void onDisconnect(BLEServer* pServer) {
      deviceConnected = false;
    }
};

void initBME280()
{
  if (!bme.begin(0x76)) 
  {
    Serial.println("Could not find a valid BME280 sensor, check wiring!");
    while (1);
  }
}

//void initBH1750()
//{
//    Full mode list of reading:
//
//      BH1750_CONTINUOUS_LOW_RES_MODE
//      BH1750_CONTINUOUS_HIGH_RES_MODE (default)
//      BH1750_CONTINUOUS_HIGH_RES_MODE_2

//      BH1750_ONE_TIME_LOW_RES_MODE
//      BH1750_ONE_TIME_HIGH_RES_MODE
//      BH1750_ONE_TIME_HIGH_RES_MODE_2
  
//  if (lightMeter.begin(BH1750::CONTINUOUS_HIGH_RES_MODE)) 
//  {
//    Serial.println(F("BH1750 Advanced begin"));
//  } 
//  else 
//  {
//    Serial.println(F("Error initialising BH1750"));
//  }
//}

void initLTR390()
{
    Serial.println("Adafruit LTR-390 test");

  if ( ! ltr390.init() ) {
    Serial.println("Couldn't find LTR sensor!");
    while (1) delay(10);
  }
  Serial.println("Found LTR sensor!");
  
  ltr390.setMode(LTR390_MODE_ALS);               //set LTR390 mode to AVS: Ambient Light Sensor 
  ltr390.setGain(LTR390_GAIN_3);                 //set LTR390 gain to 3; see example from library
  ltr390.setResolution(LTR390_RESOLUTION_18BIT); //set LTR390 resolution to 16bit;    
}

void setup() {
  // Start serial communication 
  Serial.begin(115200);
  Serial.print("Initializare port serial...");

  // Init BME280 environmental sensor
  initBME280();

  // Init BH1750 light sensor:
//  initBH1750();

  // Init LTR390:
  initLTR390();
  
  // Init onboard led:
  pinMode(led, OUTPUT);
  
  // Create the BLE Device
  BLEDevice::init("ESP32");
//  BLEDevice::init(bleServerName);
  
  // Create the BLE Server
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);
  // Create a BLE Characteristic
  pCharacteristic_bme = pService->createCharacteristic(
                      CHARACTERISTIC_UUID_bme,
                      BLECharacteristic::PROPERTY_READ  |
                      BLECharacteristic::PROPERTY_WRITE |
                      BLECharacteristic::PROPERTY_NOTIFY
                    );                   

  pCharacteristic_light = pService->createCharacteristic(
                      CHARACTERISTIC_UUID_light,
                      BLECharacteristic::PROPERTY_READ  |
                      BLECharacteristic::PROPERTY_WRITE |                      
                      BLECharacteristic::PROPERTY_NOTIFY
                    );  
 
  // Create a BLE Descriptor
  
  pDescr_bme = new BLEDescriptor((uint16_t)0x2901);
  pDescr_bme->setValue("Read data from BME280");
  pCharacteristic_bme->addDescriptor(pDescr_bme);

  pDescr_light = new BLEDescriptor((uint16_t)0x2901);
//  pDescr_light->setValue("Read data from BH1750");
  pDescr_light->setValue("Read data from LTR390");
  pCharacteristic_light->addDescriptor(pDescr_light);

  
  pBLE2902_bme = new BLE2902();
  pBLE2902_bme->setNotifications(true);
  pCharacteristic_bme->addDescriptor(pBLE2902_bme);

  pBLE2902_light = new BLE2902();
  pBLE2902_light->setNotifications(true);
  pCharacteristic_light->addDescriptor(pBLE2902_light);  
  
  // Start the service
  pService->start();

  // Start advertising
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->setScanResponse(false);
  pAdvertising->setMinPreferred(0x0);  // set value to 0x00 to not advertise this parameter
  BLEDevice::startAdvertising();
  Serial.println("Waiting a client connection to notify...");
}


void loop() 
{
  // notify changed value
  if (deviceConnected) 
  {
//**********Turn on the onboard led:
    digitalWrite(led, HIGH);

//**********Read BME280 data:   
    temp = bme.readTemperature();            // Read temperature as Celsius (the default)
    humi = bme.readHumidity();               // Read humidity
    pres = bme.readPressure() / 100.0F;      // Read atmosfperic pressure in hPa

      Serial.println("**********BME280 data:");
    
      Serial.print("Temperature Celsius: "); // print temperature in serial console
      Serial.print(temp);
      Serial.println(" C");
      
      Serial.print("Humidity: ");            // print temperature in serial console
      Serial.print(humi);
      Serial.println(" %");

      Serial.print("Pressure: ");            // print atmospheric pressure in serial console
      Serial.print(pres);
      Serial.println(" hPa");           

      Serial.println("");

//**********Read BH1750 data: 
//    if (lightMeter.measurementReady()) 
//    {
//      lux = lightMeter.readLightLevel();     // read light value in lux
//    }
//    static char lightTemp[6];                // Convert data from sensor in strings:
//    dtostrf(lux, 6, 2, lightTemp);           
//    Serial.print("Light: ");                 // print light value in serial console  
//    Serial.print(lux);
//    Serial.println(" lx");
  
//    Serial.println("");

//**********Read LTR390 data:
//    lux = ltr390.getLux();
//    uv  = ltr390.getUVI();
    
  if (ltr390.newDataAvailable()) 
  {

    Serial.println("**********LTR390 data:");
/*    
    //read Ambient light lux:
    ltr390.setMode(LTR390_MODE_ALS);                 //set sensor in ALS mode = Ambient Light Sensor
    ltr390.setGain(LTR390_GAIN_3);                   //Recommended for Lux - x3
    ltr390.setResolution(LTR390_RESOLUTION_18BIT);   //Recommended for Lux - 18-bit    
      if (ltr390.getMode() == LTR390_MODE_ALS) 
        {  
          Serial.print("Ambient Light Lux: ");
          lux = ltr390.getLux(); 
          Serial.println(lux);
        }
    delay(10);
    //read UV sensor:
    ltr390.setMode(LTR390_MODE_UVS);                 //set sensor in UV mode
    ltr390.setGain(LTR390_GAIN_18);                  //Recommended for UVI - x18
    ltr390.setResolution(LTR390_RESOLUTION_20BIT);   //Recommended for UVI - 20-bit
      if (ltr390.getMode() == LTR390_MODE_UVS) 
        {  
          Serial.print("UV Index: ");
          uv  = ltr390.getUVI();  
          Serial.println(uv);
        }   
*/

      if (ltr390.getMode() == LTR390_MODE_ALS) 
      {
         Serial.print("Ambient Light Lux: ");
         lux = ltr390.getLux(); 
//         Serial.println(ltr390.getLux());
         Serial.println(lux);
         ltr390.setGain(LTR390_GAIN_18);                  //Recommended for UVI - x18
         ltr390.setResolution(LTR390_RESOLUTION_20BIT);   //Recommended for UVI - 20-bit
         ltr390.setMode(LTR390_MODE_UVS);             
      } 
      else if (ltr390.getMode() == LTR390_MODE_UVS) 
      {
         Serial.print("UV Index: ");
         uv  = ltr390.getUVI(); 
//         Serial.println(ltr390.getUVI());
         Serial.println(uv);
         ltr390.setGain(LTR390_GAIN_3);                   //Recommended for Lux - x3
         ltr390.setResolution(LTR390_RESOLUTION_18BIT);   //Recommended for Lux - 18-bit
         ltr390.setMode(LTR390_MODE_ALS);
      }

  }

    Serial.println("");    
    
//**********Create strings used to transmit data over BLE:
    static char temperatureCTemp[6];         
    static char humidityTemp[6];          
    static char pressureAtmTemp[8]; 

    static char lightTemp[8];
    static char UVTemp[4];
    
    dtostrf(temp, 6, 2, temperatureCTemp);   // Convert data from sensor in string
    dtostrf(humi, 6, 2, humidityTemp);       // Convert data from sensor in string
    dtostrf(pres, 6, 2, pressureAtmTemp);    // Convert data from sensor in string

    dtostrf(lux, 8, 2, lightTemp);
    dtostrf(uv,  2, 2, UVTemp);
    
//**********Create combined strings:
    char bmeCombinedString[6 * 3 + 2 + 1]; // Space for 3 strings + 2 x "," + null end character 
    char lightCombinedString[(8+2) + 1 + 1];       // Space for 2 x strings + 1 x "," + null end character
            
//**********Concatenate of strings:

    strcpy(bmeCombinedString, temperatureCTemp);
    strcat(bmeCombinedString, ",");    
    strcat(bmeCombinedString, humidityTemp);
    strcat(bmeCombinedString, ",");     
    dtostrf(pres, 6, 2, pressureAtmTemp);    // Convert data from sensor in string
    strcat(bmeCombinedString, pressureAtmTemp);

    dtostrf(lux, 8, 2, lightTemp);
    strcpy(lightCombinedString, lightTemp);
    strcat(lightCombinedString, ",");
    strcat(lightCombinedString, UVTemp);  

    pCharacteristic_bme->setValue(bmeCombinedString);
//    pCharacteristic_temp->notify();

//    pCharacteristic_light->setValue(lightTemp);
    pCharacteristic_light->setValue(lightCombinedString);
//    pCharacteristic_humi->notify();
   
    delay(10000);
    }
    // disconnecting
    if (!deviceConnected && oldDeviceConnected) 
    {
        digitalWrite(led, LOW);
        delay(500);                   // give the bluetooth stack the chance to get things ready
        pServer->startAdvertising();  // restart advertising
        Serial.println("start advertising");
        oldDeviceConnected = deviceConnected;
    }
    // connecting
    if (deviceConnected && !oldDeviceConnected) 
    {
        // do stuff here on connecting
        oldDeviceConnected = deviceConnected;
    }
}
