import sqlite3
import pandas as pd
import joblib
import requests
import sys
from statistics import mode
from datetime import datetime, timedelta

#load model:
#model = joblib.load('model.pkl')
model = joblib.load("/userdata/For_ML/model.pkl")

#connect to sqlite database:
conn = sqlite3.connect('/userdata/sqlite/sensor_data_1.db')

#calculate interval for 2 minutes, from database:
end_time = datetime.now()
start_time = end_time - timedelta(minutes=2)

#format timestamps:
start_time_str = start_time.strftime('%Y-%m-%d %H:%M')
end_time_str = end_time.strftime('%Y-%m-%d %H:%M')

#query data from the last 1 minutes:
query = """
SELECT temperature, humidity, pressure, uv_level, light
FROM readings
WHERE timestamp BETWEEN ? AND ?
"""

#params = (start_time.strftime('%Y-%m-%d %H:%M:%S'), end_time.strftime('%Y-%m-%d %H:%M:%S')
params = (start_time_str, end_time_str)

#read data in dataFrame:
df = pd.read_sql_query(query, conn, params=params)

#check if data is abailable:
if df.empty:
	print("No data is avalable in last 2 minutes.")
	#print(start_time_str)
	#print(end_time_str)
else:
	print(df)
	#prepare data for prediction:
	input_data = df[['temperature', 'humidity', 'pressure', 'uv_level','light']]
	
	#prediction:
	predictions = model.predict(input_data)
	
	#display recommendations and send them to Node-red:
	for index, recomandation in enumerate(predictions):
		print(f"Recommendation for data from index {index}: {recomandation}")
	
	recomandation = int(recomandation)
	#final_recommandation = mode(predictions) #use statistic "'majirity of votes" to see what is predominant remmendation from readings
	
	#create payload to be sent to Node-red:
	payload = {'Recommandation': recomandation}
	#payload = {'Recommandation': int(final_recommandation)}	 #send the predominant recommendation to Node-Red
	#Node-Red endpoint URL:
	url = 'http://localhost:1880/recommandation'
	
	#send data to Node-red:
	try:
		response = requests.post(url, json=payload, timeout=5)
		response.raise_for_status()
		if response.status_code ==200:
			print(f"Recomendation was sent with success to Node-Red: {payload}")
		else:
			print(f"Error to send recommendation to Node-Red: {response.status_code}")
	except Exception as e:
		print(f"There was an error sending the recommendation: {e}")
		
#close connection with database:
conn.close()

#end script:
sys.exit()
