
#import sys
#sys.path.append('/userdata/For_ML')

#1. import all necessaryes libraries:
import sqlite3
import pandas as pd
import numpy as np
import joblib

from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.preprocessing import StandardScaler
from sklearn.metrics import accuracy_score
from sklearn.model_selection import cross_val_score

#2. connect to database and load data:
conn = sqlite3.connect('/userdata/sqlite/sensor_data_1.db') 
query = "SELECT temperature, humidity, pressure, uv_level, light, recommandation FROM recommandations"
data = pd.read_sql_query(query, conn)

#3. prepare data for training:
#transform column 'recommendation' in numeric value (factorize the text)
data['recommandation_encoded'] = pd.factorize(data['recommandation'])[0]

#select inut data (x) and target (y)
x = data[['temperature','humidity','pressure','uv_level','light']]
y = data['recommandation_encoded']

#split data set in dataset for training and dataset for test (80% training, 20% test)
x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=0.2, random_state=42)

#scale the data using StandardScaler:
scaler = StandardScaler()
x_train_scaled = scaler.fit_transform(x_train)
x_test_scaled = scaler.transform(x_test)

#4. create and train model logistic regression:
model = LogisticRegression(max_iter=2000)
model.fit(x_train, y_train)

#5. evaluate the model:
y_pred = model.predict(x_test)
accuracy = accuracy_score(y_test, y_pred)

print(f'Model accuracy: {accuracy * 100:.2f}%')

scores = cross_val_score(model, x_train, y_train, cv=5)
print("Cross-validation scores: ", scores)
print("Average cross-validation score: ", scores.mean())

#6. save the model:
joblib.dump(model, 'model.pkl')
print("The model was saved with success!")

#close connection with database:
conn.close()
